@@ -25,7 +25,7 @@
 setClass(
   Class = "webgeom",
   prototype = prototype(
-    url = "https://cida.usgs.gov/gdp/geoserver/wfs",
+    url = paste0(geoserver_base(), "/wfs"),
     geom = as.character(NA), 
     attribute = as.character(NA),
     values = as.character(NA), 
@@ -43,8 +43,6 @@ setClass(
     GML_SCHEMA_LOCATION = "character")
 )
 
-
-
 setMethod("initialize", signature = "webgeom",
           definition = function(
             .Object, url = .Object@url, geom = .Object@geom, 
@@ -106,8 +104,7 @@ setMethod("initialize", signature = "webgeom",
 #' #2) locate the \code{attribute} for county names by looking at the options for attributes
 #' query(webgeom(geom='sample:Counties'), 'attributes') # discover FIPS
 #' #3) find the appropriate fip code for the county:
-#' howard.fips <- maps::county.fips %>% 
-#'    dplyr::filter(polyname == 'texas,howard') %>% .$fips %>% as.character
+#' howard.fips <- "48227"
 #' #4) create a webgeom for the Howard County in Texas
 #' stencil <- webgeom(geom='sample:Counties', attribute='FIPS', values=howard.fips)
 #' #5) get data for Howard County
